"""
  New job class to run Prod6 for Paranal and La Palma
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class Prod6MCPipeNSBJob(Job):
    """Job extension class for Prod5 MC NSB simulations,
    takes care of running corsika piped into simtel
    2 output files are created for Dark and Moon NSB
    """

    def __init__(self, cpu_time=259200):
        """Constructor takes almosst everything from base class

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpu_time)
        self.setName("Prod6MC_Generation")
        self.setType("MCSimulation")
        self.package = "corsika_simtelarray"
        self.version = "2022-08-03"
        self.compiler = "gcc83_matchcpu"
        self.program_category = "tel_sim"
        self.prog_name = "sim_telarray"
        self.configuration_id = 15
        self.output_data_level = DATA_LEVEL_METADATA_ID["R1"]
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD6/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.metadata = collections.OrderedDict()

    def set_site(self, site):
        """Set the site to simulate

        Parameters:
        site -- a string for the site name (LaPalma)
        """
        if site in ["Paranal", "LaPalma"]:
            DIRAC.gLogger.info(f"Set Corsika site to: {site}")
            self.cta_site = site
        else:
            DIRAC.gLogger.error(f"Site is unknown: {site}")
            DIRAC.exit(-1)

    def set_particle(self, particle):
        """Set the corsika primary particle

        Parameters:
        particle -- a string for the particle type/name
        """
        if particle in ["gamma", "gamma-diffuse", "electron", "proton", "helium"]:
            DIRAC.gLogger.info(f"Set Corsika particle to: {particle}")
            self.particle = particle
        else:
            DIRAC.gLogger.error(f"Corsika does not know particle type: {particle}")
            DIRAC.exit(-1)

    def set_pointing_dir(self, pointing):
        """Set the pointing direction, North or South

        Parameters:
        pointing -- a string for the pointing direction
        """
        if pointing in ["North", "South", "East", "West"]:
            DIRAC.gLogger.info(f"Set Pointing dir to: {pointing}")
            self.pointing_dir = pointing
        else:
            DIRAC.gLogger.error(f"Unknown pointing direction: {pointing}")
            DIRAC.exit(-1)

    def set_full_moon(self, full_moon=False):
        """Set if to simulate with full-moon conditions

        Parameters:
        full_moon -- a boolean for simulating with full moon conditions
        """
        if full_moon is True:
            DIRAC.gLogger.info("Set simulations with full-moon conditions")
            self.full_moon = "--with-full-moon"
        else:
            self.full_moon = "--with-halfmoon"  # We alawys at least run with half moon

    def set_magic(self, with_magic=False):
        """Set if to simulate with MAGIC

        Parameters:
        with_magic -- a boolean for simulating with MAGIC
        """
        if with_magic is True:
            DIRAC.gLogger.info("Set simulations with MAGIC telescopes")
            self.with_magic = "--with-magic"
        else:
            self.with_magic = ""

    def set_sct(self, with_sct=False):
        """Set if to include SCTs in simulations

        Parameters:
        with_sct -- a boolean if to include SCTs
        """
        if with_sct is True:
            DIRAC.gLogger.info("Set to include SCTs")
            self.with_sct = "--with-sct"
            self.sct_metadata = "True"
        else:
            self.with_sct = ""
            self.sct_metadata = "False"

    def set_meta_data(self):
        """define the common meta data of the application"""
        # The order of the metadata dictionary is important,
        # since it's used to build the directory structure
        self.metadata["array_layout"] = "Prod6-Hyperarray"
        self.metadata["site"] = self.cta_site
        self.metadata["particle"] = self.particle
        # for air shower simulation means North=0 and South=180
        # but here piped into tel_sim so North=180 and South=0
        if self.pointing_dir == "North":
            self.metadata["phiP"] = 180
        if self.pointing_dir == "South":
            self.metadata["phiP"] = 0
        self.metadata["thetaP"] = float(self.zenith_angle)
        self.metadata[self.program_category + "_prog"] = self.prog_name
        self.metadata[self.program_category + "_prog_version"] = "".join(
            self.version.rsplit("-sc", 1)
        )
        self.metadata["data_level"] = self.output_data_level
        self.metadata["configuration_id"] = self.configuration_id

    def setupWorkflow(self, debug=False):
        """Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
        """
        # step 1 - debug only
        i_step = 1
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2 : use new CVMFS repo
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run corsika+sim_telarray
        prod_exe = "./dirac_prod6_run"
        prod_args = "--start_run {} --run {} {} {} {} {} {} {} {}".format(
            self.start_run_number,
            self.run_number,
            self.full_moon,
            self.with_sct,
            self.with_magic,
            self.cta_site,
            self.particle,
            self.pointing_dir,
            self.zenith_angle,
        )

        cs_step = self.setExecutable(
            prod_exe, arguments=prod_args, logFile="CorsikaSimtel_Log.txt"
        )
        cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
        cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"
        i_step += 1

        # step 4 verify the number of events in the simtel file
        data_output_pattern = "Data/*.simtel.zst"
        mgv_step = self.setExecutable(
            "dirac_simtel_check",
            arguments=f"'{data_output_pattern}'",
            logFile="Verify_n_showers_Log.txt",
        )
        mgv_step["Value"]["name"] = "Step%i_VerifyNShowers" % i_step
        mgv_step["Value"]["descr_short"] = "Verify number of showers"
        i_step += 1

        # step 5 - define meta data, upload file on SE and register in catalogs
        self.set_meta_data()
        md_json = json.dumps(self.metadata)

        # Upload and register data - NSB=1 dark
        file_meta_data = {
            "runNumber": self.run_number,
            "nsb": 1,
            "sct": self.sct_metadata,
        }
        file_md_json = json.dumps(file_meta_data)
        data_output_pattern = "Data/*dark*.simtel.zst"

        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                md_json,
                file_md_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_dark_Log.txt",
        )
        dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # Upload and register log and histo file - NSB=1
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)
        log_file_pattern = "Data/*dark*.log_hist.tar"
        log_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' %s '%s' %s %s '%s' Log"
            % (
                md_json,
                file_md_json,
                self.base_path,
                log_file_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="LogManagement_dark_Log.txt",
        )
        log_step["Value"]["name"] = f"Step{i_step}_LogManagement"
        log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
        i_step += 1

        # Now switching to half moon NSB
        # Upload and register data - NSB=5 half moon
        file_meta_data = {
            "runNumber": self.run_number,
            "nsb": 5,
            "sct": self.sct_metadata,
        }
        file_md_json = json.dumps(file_meta_data)
        data_output_pattern = "Data/*-moon*.simtel.zst"

        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                md_json,
                md_field_json,
                file_md_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_moon_Log.txt",
        )
        dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # Upload and register log file - NSB=5
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)
        log_file_pattern = "Data/*-moon*.log_hist.tar"
        log_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
            % (
                md_json,
                md_field_json,
                file_md_json,
                self.base_path,
                log_file_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="LogManagement_moon_Log.txt",
        )
        log_step["Value"]["name"] = f"Step{i_step}_LogManagement"
        log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
        i_step += 1

        if self.full_moon == "--with-full-moon":
            # Now switching to full moon NSB
            # Upload and register data - NSB=19 full moon
            file_meta_data = {
                "runNumber": self.run_number,
                "nsb": 19,
                "sct": self.sct_metadata,
            }
            file_md_json = json.dumps(file_meta_data)
            data_output_pattern = "Data/*-fullmoon*.simtel.zst"

            dm_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
                % (
                    md_json,
                    md_field_json,
                    file_md_json,
                    self.base_path,
                    data_output_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile="DataManagement_fullmoon_Log.txt",
            )
            dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
            dm_step["Value"][
                "descr_short"
            ] = "Save data files to SE and register them in DFC"
            i_step += 1

            # Upload and register log file - NSB=19
            file_meta_data = {}
            file_md_json = json.dumps(file_meta_data)
            log_file_pattern = "Data/*-fullmoon*.log_hist.tar"
            log_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
                % (
                    md_json,
                    md_field_json,
                    file_md_json,
                    self.base_path,
                    log_file_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile="LogManagement_fullmoon_Log.txt",
            )
            log_step["Value"]["name"] = f"Step{i_step}_LogManagement"
            log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
            i_step += 1

        # Step 6 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = f"Step{i_step}_LSHOME_End"
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv({"NSHOW": f"{self.n_shower}"})



#!/usr/bin/env python
"""
Add a dataset from a query specified in a json file, e.g.:

{"MCCampaign": "PROD5b",
"particle": "proton",
"site": "Paranal"}

The created dataset will get the name of the json file (without the extension)

Usage:
   cta-prod-add-dataset <json file with dataset query dict>
Arguments:
   json file with dataset query dict
   The file should have the .json extension, e.g. my_dataset.json
"""

__RCSID__ = "$Id$"

import json
import ast
import os

import DIRAC
from DIRAC.Core.Base.Script import Script
from DIRAC import gLogger
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


@Script()
def main():
    fc = FileCatalogClient()
    Script.parseCommandLine(ignoreErrors=True)
    argss = Script.getPositionalArgs()
    if len(argss) == 1:
        inputJson = argss[0]
        if not inputJson.endswith(".json"):
            Script.showHelp()
    else:
        Script.showHelp()

    # read the meta data query from json file
    content = open(inputJson).readlines()
    for line in content:
        json_string = json.dumps(ast.literal_eval(line))

    MDdict = json.loads(json_string)

    # ensure the integer values are written in the right format
    for key, value in MDdict.items():
        if isinstance(value, float):
            MDdict[key] = {"=": value}

    datasetName = os.path.basename(inputJson).split(".json")[0]
    datasetPath = os.path.join("/vo.cta.in2p3.fr/datasets/", datasetName)
    res = fc.addDataset({datasetPath: MDdict})

    if not res["OK"]:
        gLogger.error(f"Failed to add dataset {datasetName}: {res['Message']}")
        DIRAC.exit(-1)

    if datasetName in res["Value"]["Failed"]:
        gLogger.error(
            "Failed to add dataset %s: %s"
            % (datasetName, res["Value"]["Failed"][datasetName])
        )
        DIRAC.exit(-1)

    gLogger.notice("Successfully added dataset", datasetName)
    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Move a service from DIPS to HTTPS using Tornado

Prereq:
The service shloud be already install.
Tornado service must be installed.
The HandlerPath must exists: DIRAC/.../*TornadoHandler.py

Usage:
cta-install-tornado-component <System> <Component> <HandlerPath>
"""
import subprocess as sp
from DIRAC import exit as DIRACexit
from DIRAC.Core.Base.Script import Script

from DIRAC.ConfigurationSystem.Client.CSAPI import CSAPI
from DIRAC import gConfig, gLogger, S_OK
from DIRAC.FrameworkSystem.Client.ComponentInstaller import gComponentInstaller
from DIRAC.Core.Utilities.Extensions import extensionsByPriority
from DIRAC.ConfigurationSystem.Client.Helpers import CSGlobals


class TornadoComponentInstaller:
    def __init__(self):
        self.setup = CSGlobals.getSetup()
        self.tornadoPort = self._getValue(f"/Systems/Tornado/{self.setup}/Port")
        self.cfgClient = CSAPI()

    def _removeOptionFromCS(self, path):
        """
        Delete options from central CS
        """
        result = self.cfgClient.downloadCSData()
        if not result["OK"]:
            return result
        result = self.cfgClient.delOption(path)
        if not result["OK"]:
            return result
        result = self.cfgClient.commit()
        return result

    def _modifyValue(self, path, new_value):
        result = self.cfgClient.downloadCSData()
        if not result["OK"]:
            return result
        result = self.cfgClient.modifyValue(path, new_value)
        if not result["OK"]:
            return result
        result = self.cfgClient.commit()
        return result

    def _setOption(self, path, value):
        result = self.cfgClient.downloadCSData()
        if not result["OK"]:
            return result
        result = self.cfgClient.setOption(path, value)
        if not result["OK"]:
            return result
        result = self.cfgClient.commit()
        return result

    def _getValue(self, path):
        value = gConfig.getValue(path)
        return value

    def _getSection(self, path):
        section = gConfig.getSections(path)
        return section

    def _getHosts(self):
        hosts = gConfig.getSections("/Registry/Hosts/")
        return hosts

    def valueExists(self, path):
        value = self._getValue(path)
        if value != None:
            return True
        else:
            return False

    def removeComponent(self, system, component, removeLogs):
        res = gComponentInstaller.runsvctrlComponent(system, component, "d")
        res = gComponentInstaller.unsetupComponent(system, component)
        if removeLogs:
            for runitCompDir in glob.glob(
                os.path.join(self.runitDir, system, component)
            ):
                try:
                    shutil.rmtree(runitCompDir)
                except Exception:
                    gLogger.exception()

    def checkTornadoHandlerPath(self, handlerPathValue):
        cmd = "pip show DIRAC | grep Location | awk -F' ' '{print $2}'"
        out = sp.getstatusoutput(cmd)
        if out[0] == 0:
            diracLocation = out[1]
        else:
            gLogger.error(out[1])

        handlerPath = f"{diracLocation}/{handlerPathValue}"
        res = sp.getstatusoutput(f"ls {handlerPath}")

        if res[0] == 0:
            gLogger.notice(f"Find HandlerPath: {handlerPathValue}")
            return True
        else:
            gLogger.error(res[1])
            return False

    def installTornadoService(self, system, service, handlerPathValue):
        res = self.checkTornadoHandlerPath(handlerPathValue)
        if not res:
            gLogger.error(f"Did not find HandlerPath")
            DIRACexit(1)

        servicePath = f"/Systems/{system}/{self.setup}/Services/{service}"
        # change Port value
        portPath = f"{servicePath}/Port"
        currentPort = self._getValue(portPath)
        if not currentPort:
            gLogger.error(f"No Port find for {system}/{service}")
            DIRACexit(1)
        if self._getValue(portPath):
            result = self._removeOptionFromCS(portPath)
            if not result["OK"]:
                gLogger.error(result["Message"])
                DIRACexit(1)
            gLogger.notice(f"Succesfully removed {system}/{service} Port value")

        # add protocol https
        protocolPath = f"{servicePath}/Protocol"
        if self._getValue(protocolPath):
            result = self._modifyValue(protocolPath, "https")
        else:
            result = self._setOption(protocolPath, "https")
        if not result["OK"]:
            gLogger.error(result["Message"])
            DIRACexit(1)
        gLogger.notice(f"Succesfully add {system}/{service} Protocol value")

        # add tornado HandlerPath
        handlerPath = f"{servicePath}/HandlerPath"
        if self._getValue(handlerPath):
            result = self._modifyValue(handlerPath, handlerPathValue)
        else:
            result = self._setOption(handlerPath, handlerPathValue)
        if not result["OK"]:
            gLogger.error(result["Message"])
            DIRACexit(1)
        gLogger.notice(f"Succesfully add {system}/{service} HandlerPath value")

        # change URL port
        urlPath = f"/Systems/{system}/{self.setup}/URLs/{service}"
        urlDIPS = self._getValue(urlPath)
        if urlDIPS:
            urlHTTPS = urlDIPS.replace("dips", "https")
            urlHTTPS = urlHTTPS.replace(currentPort, self.tornadoPort)
            result = self._modifyValue(urlPath, urlHTTPS)
        else:
            hosts = self._getHosts()
            host = hosts[0]
            urlHTTPS = f"https://{host}:{self.tornadoPort}/{system}/{service}"
            result = self._setOption(urlPath, urlHTTPS)
        if not result["OK"]:
            gLogger.error(result["Message"])
            DIRACexit(1)
        gLogger.notice(f"Succesfully change {system}/{service} URL value")

        # remove component:
        self.removeComponent(system, service, False)
        gLogger.notice(f"Succesfully remove {system}/{service} startup and runit")
        # restart tornado service:
        result = gComponentInstaller.runsvctrlComponent("Tornado", "Tornado", "t")
        gLogger.notice(f"Succesfully restart Tornado service")


@Script()
def main():
    Script.parseCommandLine(ignoreErrors=True)

    TornadoInstaller = TornadoComponentInstaller()
    gComponentInstaller.exitOnError = True

    setOverwrite = False
    setSpecialOption = False
    Script.registerSwitch(
        "w", "overwrite", "Overwrite the configuration in the global CS", setOverwrite
    )
    Script.registerSwitch(
        "p:", "parameter=", "Special component option ", setSpecialOption
    )

    # Registering arguments will automatically add their description to the help menu

    Script.registerArgument(
        (
            "System/Component: Full component name (ie: WorkloadManagement/Matcher)",
            "System:           Name of the DIRAC system (ie: WorkloadManagement)",
        )
    )
    Script.registerArgument(
        " HandlerPath:        HandlerPath (i.e DIRAC/.../TornadoHandler)",
        mandatory=True,
    )
    Script.registerArgument(
        " Component:        Name of the DIRAC service (ie: Matcher)", mandatory=False
    )

    Script.parseCommandLine()
    args = Script.getPositionalArgs()

    if len(args) == 2:
        args = args[0].split("/") + args[1]

    if len(args) != 3:
        Script.showHelp(exitCode=1)
    system = args[0]
    component = args[1]
    handlerPath = args[2]

    result = gComponentInstaller.getSoftwareComponents(extensionsByPriority())
    if not result["OK"]:
        gLogger.error(result["Message"])
        DIRACexit(1)
    availableComponents = result["Value"]

    for compType in availableComponents:
        if (
            system in availableComponents[compType]
            and component in availableComponents[compType][system]
        ):
            cType = compType[:-1].lower()
            break
    else:
        gLogger.error(
            f"Component {system}/{component} is not available for installation"
        )
        DIRACexit(1)

    TornadoInstaller.installTornadoService(system, component, handlerPath)


if __name__ == "__main__":
    main()



"""
  New job class to run Prod6 Divergent simulations
  This is similar to the parallel Prod6 simulations and inherits from it but with a divergent option.
  The divergent angles are hardcoded everywhere and also here.
          OG, Dec 2022
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Interfaces.API.Prod6MCPipeNSBJob import Prod6MCPipeNSBJob


class Prod6MCPipeDivergentJob(Prod6MCPipeNSBJob):
    """Job extension class for Prod6 MC Divergent simulations"""

    def __init__(self, cpu_time=259200):
        """Constructor takes almosst everything from base class

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Prod6MCPipeNSBJob.__init__(self, cpu_time)
        self.catalogs = json.dumps(["DIRACFileCatalog"])
        self.metadata = collections.OrderedDict()

    def set_half_moon(self, half_moon=False):
        """Set if to simulate with half-moon conditions

        Parameters:
        half_moon -- a boolean for simulating with full moon conditions
        """
        if half_moon is True:
            gLogger.info("Set simulations with half-moon conditions")
            self.half_moon = "--with-half-moon"
        else:
            self.half_moon = ""  # We alawys at least run with half moon

    def setupWorkflow(self, debug=False):
        """Override the base class job workflow to adapt to divergent simulations
        All parameters shall have been defined before that method is called.
        """
        # step 1 - debug only
        i_step = 1
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2 : use new CVMFS repo
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1
        # step 3 run corsika+sim_telarray
        prod_exe = "./dirac_prod6_run"
        prod_args = (
            "--start_run %s --run %s --sequential --divergent %s %s %s %s %s %s %s"
            % (
                self.start_run_number,
                self.run_number,
                self.half_moon,
                self.with_sct,
                self.with_magic,
                self.cta_site,
                self.particle,
                self.pointing_dir,
                self.zenith_angle,
            )
        )

        cs_step = self.setExecutable(
            prod_exe, arguments=prod_args, logFile="CorsikaSimtel_Log.txt"
        )
        cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
        cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"
        i_step += 1

        # step 4 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_End" % i_step
            ls_step["Value"][
                "descr_short"
            ] = "list files in working directory and sub-directory"
            i_step += 1

        # step 5  verify the number of events in the simtel file and define meta data, upload file on SE and register in catalogs
        self.set_meta_data()
        md_json = json.dumps(self.metadata)

        i_substep = 0
        for div_ang in ["0.0022", "0.0043", "0.008", "0.01135", "0.01453"]:
            data_output_pattern = f"Data/*-div{div_ang}*-dark*.simtel.zst"

            # verify the number of events in the simtel file
            mgv_step = self.setExecutable(
                "dirac_simtel_check",
                arguments=f"'{data_output_pattern}'",
                logFile=f"Verify_n_showers_div{div_ang}_Log.txt",
            )
            mgv_step["Value"]["name"] = f"Step{i_step}.{i_substep}_VerifyNShowers"
            mgv_step["Value"]["descr_short"] = "Verify number of showers"

            i_substep += 1

            # define meta data, upload file on SE and register in catalogs

            # Upload and register data
            file_meta_data = {
                "runNumber": self.run_number,
                "nsb": 1,
                "div_ang": div_ang,
            }  # adding a new meta data field
            file_md_json = json.dumps(file_meta_data)

            dm_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
                % (
                    md_json,
                    file_md_json,
                    self.base_path,
                    data_output_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"DataManagement_div{div_ang}_Log.txt",
            )
            dm_step["Value"]["name"] = f"Step{i_step}.{i_substep}_DataManagement"
            dm_step["Value"][
                "descr_short"
            ] = "Save data files to SE and register them in DFC"

            i_substep += 1

            # Upload and register log_and_hist file
            file_meta_data = {}
            file_md_json = json.dumps(file_meta_data)
            log_file_pattern = f"Data/*-div{div_ang}*-dark*.log_hist.tar"
            log_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' %s '%s' %s %s '%s' Log"
                % (
                    md_json,
                    file_md_json,
                    self.base_path,
                    log_file_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"LogManagement_div{div_ang}_Log.txt",
            )
            log_step["Value"]["name"] = f"Step{i_step}.{i_substep}_LogManagement"
            log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"

            i_substep += 1

            # Now switching to half moon NSB
            if self.half_moon:
                # Upload and register data - NSB=5 half moon
                file_meta_data = {
                    "runNumber": self.run_number,
                    "nsb": 5,
                    "div_ang": div_ang,
                }
                file_md_json = json.dumps(file_meta_data)
                data_output_pattern = f"Data/*-div{div_ang}*-moon*.simtel.zst"
                # verify the number of events in the simtel file
                mgv_step = self.setExecutable(
                    "dirac_simtel_check",
                    arguments=f"'{data_output_pattern}'",
                    logFile=f"Verify_n_showers_moon_div{div_ang}_Log.txt",
                )
                mgv_step["Value"]["name"] = f"Step{i_step}.{i_substep}_VerifyNShowers"
                mgv_step["Value"]["descr_short"] = "Verify number of showers"

                i_substep += 1

                dm_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
                    % (
                        md_json,
                        file_md_json,
                        self.base_path,
                        data_output_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile=f"DataManagement_moon_div{div_ang}_Log.txt",
                )
                dm_step["Value"]["name"] = f"Step{i_step}.{i_substep}_DataManagement"
                dm_step["Value"][
                    "descr_short"
                ] = "Save data files to SE and register them in DFC"

                i_substep += 1

                # Upload and register log file - NSB=5
                file_meta_data = {}
                file_md_json = json.dumps(file_meta_data)
                log_file_pattern = f"Data/*-div{div_ang}*-moon*.log_hist.tar"
                log_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' %s '%s' %s %s '%s' Log"
                    % (
                        md_json,
                        file_md_json,
                        self.base_path,
                        log_file_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile=f"LogManagement_moon_div{div_ang}_Log.txt",
                )
                log_step["Value"]["name"] = f"Step{i_step}.{i_substep}_LogManagement"
                log_step["Value"][
                    "descr_short"
                ] = "Save log to SE and register them in DFC"

                i_substep += 1

        i_step += 1

        # Step 6 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = f"Step{i_step}_LSHOME_End"
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            # i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv({"NSHOW": f"{self.n_shower}"})



"""
  New job class to run Prod6 for Paranal and La Palma
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
import itertools


# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class MCPipeNSBJob(Job):
    """Job extension class for Prod5 MC NSB simulations,
    takes care of running corsika piped into simtel
    2 output files are created for Dark and Moon NSB
    """

    def __init__(self, cpu_time=259200):
        """Constructor takes almosst everything from base class

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        super().__init__()
        self.setCPUTime(cpu_time)
        self.setName("MC_Generation")
        self.setType("MCSimulation")
        self.package = "corsika_simtelarray"
        self.version = "2022-08-03"
        self.compiler = "gcc83_matchcpu"
        self.program_category = "tel_sim"
        self.prog_name = "sim_telarray"
        self.start_run_number = "0"
        self.run_number = "@{JOB_ID}"  # dynamic
        self.configuration_id = 15
        self.output_data_level = DATA_LEVEL_METADATA_ID["R1"]
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD6/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.output_metadata = collections.OrderedDict()
        self.output_file_metadata = collections.OrderedDict()
        self.sct = ""
        self.magic = ""
        self.moon = ""
        self.array_layout = "Prod6-Hyperarray"
        self.div_ang = ""

    def set_site(self, site):
        """Set the site to simulate

        Parameters:
        site -- a string for the site name (LaPalma)
        """
        if site in ["Paranal", "LaPalma"]:
            DIRAC.gLogger.info(f"Set Corsika site to: {site}")
            self.site = site
        else:
            DIRAC.gLogger.error(f"Site is unknown: {site}")
            DIRAC.exit(-1)

    def set_particle(self, particle):
        """Set the corsika primary particle

        Parameters:
        particle -- a string for the particle type/name
        """
        if particle in ["gamma", "gamma-diffuse", "electron", "proton", "helium"]:
            DIRAC.gLogger.info(f"Set Corsika particle to: {particle}")
            self.particle = particle
        else:
            DIRAC.gLogger.error(f"Corsika does not know particle type: {particle}")
            DIRAC.exit(-1)

    def set_pointing_dir(self, pointing):
        """Set the pointing direction, North or South

        Parameters:
        pointing -- a string for the pointing direction
        """
        if pointing in ["North", "South", "East", "West"]:
            DIRAC.gLogger.info(f"Set Pointing dir to: {pointing}")
            self.pointing_dir = pointing
        else:
            DIRAC.gLogger.error(f"Unknown pointing direction: {pointing}")
            DIRAC.exit(-1)

    def set_moon(self, moon=["dark", "half", "full"]):
        """Set to simulate with various moon conditions

        Parameters:
        moon -- a list of moon conditions for simulation
        """
        if moon == ["dark"]:
            DIRAC.gLogger.info("Set simulations with dark conditions")
            self.moon = ""
            self.output_file_metadata["nsb"] = [1]
        elif moon == ["dark", "half"]:
            DIRAC.gLogger.info("Set simulations with half-moon conditions")
            self.moon = "--with-half-moon"
            self.output_file_metadata["nsb"] = [1, 5]

        elif moon == ["dark", "half", "full"]:
            DIRAC.gLogger.info("Set simulations with full-moon conditions")
            self.moon = "--with-full-moon"
            self.output_file_metadata["nsb"] = [1, 5, 19]
        else:
            DIRAC.gLogger.error(
                "Unknown moon option: %s. Options for simulation step are: \n [dark] \n [dark, half] \n [dark, half, "
                "full] " % str(moon).replace("'", "")
            )
            DIRAC.exit(-1)

    def set_div_ang(self, div_ang=None):
        div_default = [
            "0.0022",
            "0.0043",
            "0.008",
            "0.01135",
            "0.01453",
        ]  # Default divergent angles
        if div_ang is not None:
            if str(div_ang).replace(", ", ",").split(sep=",") != div_default:
                DIRAC.gLogger.error(
                    "Unknown div_ang option: %s. Option for simulation step is: %s"
                    % (div_ang, ", ".join(str(x) for x in div_default))
                )
                DIRAC.exit(-1)
            else:
                self.div_ang = div_default
                self.output_file_metadata["div_ang"] = div_default

    def set_magic(self, with_magic=False):
        """Set to simulate with MAGIC

        Parameters:
        with_magic -- a boolean for simulating with MAGIC
        """
        if with_magic is True:
            DIRAC.gLogger.info("Set simulations with MAGIC telescopes")
            self.magic = "--with-magic"

    def set_sct(self, with_sct=None):
        """Set to include SCTs in simulations

        Parameters:
        with_sct -- a string to include SCTs
        """
        if with_sct is not None:
            if with_sct.lower() == "all":
                DIRAC.gLogger.info("Set to include SCTs for all MST positions")
                self.sct = "--with-all-scts"
            elif with_sct.lower() == "non-alpha":
                DIRAC.gLogger.info("Set to include SCTs for non-Alpha MST positions")
                self.sct = "--with-sct"
            self.version = self.version + "-sc"

    def set_output_metadata(self):
        """define the common meta data of the application"""
        # The order of the metadata dictionary is important,
        # since it's used to build the directory structure
        self.output_metadata["array_layout"] = self.array_layout
        self.output_metadata["site"] = self.site
        self.output_metadata["particle"] = self.particle
        # for air shower simulation means North=0 and South=180
        # but here piped into tel_sim so North=180 and South=0
        if self.pointing_dir == "North":
            self.output_metadata["phiP"] = 180
        if self.pointing_dir == "South":
            self.output_metadata["phiP"] = 0
        self.output_metadata["thetaP"] = float(self.zenith_angle)
        if self.sct:
            self.output_metadata["sct"] = "True"
        else:
            self.output_metadata["sct"] = "False"
        self.output_metadata[self.program_category + "_prog"] = self.prog_name
        self.output_metadata[self.program_category + "_prog_version"] = "".join(
            self.version.rsplit("-sc", 1)
        )
        self.output_metadata["data_level"] = self.output_data_level
        self.output_metadata["configuration_id"] = self.configuration_id
        self.output_metadata["merged"] = 0
        self.output_metadata["MCCampaign"] = self.MCCampaign

    def set_executable_sequence(self, debug=False):
        """Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
        """
        # step 1 - debug only
        i_step = 1
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2 - use new CVMFS repo
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 - run corsika+sim_telarray
        prod_exe = "./dirac_prod_run"
        if self.div_ang:
            prod_args = (
                "--start_run %s --run %s --sequential --divergent %s %s %s %s %s %s %s"
                % (
                    self.start_run_number,
                    self.run_number,
                    self.moon,
                    self.sct,
                    self.magic,
                    self.site,
                    self.particle,
                    self.pointing_dir,
                    self.zenith_angle,
                )
            )

        else:
            prod_args = "--start_run {} --run {} {} {} {} {} {} {} {}".format(
                self.start_run_number,
                self.run_number,
                self.moon,
                self.sct,
                self.magic,
                self.site,
                self.particle,
                self.pointing_dir,
                self.zenith_angle,
            )

        cs_step = self.setExecutable(
            prod_exe, arguments=prod_args, logFile="CorsikaSimtel_Log.txt"
        )
        cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
        cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"
        i_step += 1

        # step 4 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_End" % i_step
            ls_step["Value"][
                "descr_short"
            ] = "list files in working directory and sub-directory"
            i_step += 1

        # step 5 - data management
        md_json = json.dumps(self.output_metadata)

        keys = self.output_file_metadata.keys()

        for element in itertools.product(*self.output_file_metadata.values()):
            i_substep = 1
            combination = dict(zip(keys, element))

            # build file meta data
            file_meta_data = {
                "runNumber": self.run_number,
            }
            for key, value in combination.items():
                file_meta_data[key] = value

            file_md_json = json.dumps(file_meta_data)

            if combination["nsb"] == 1:
                moon_str = "dark"
            if combination["nsb"] == 5:
                moon_str = "moon"
            if combination["nsb"] == 19:
                moon_str = "fullmoon"

            if combination.get("div_ang"):
                div = f"div{combination['div_ang']}"
                div_str = div + "_"
            else:
                div = ""
                div_str = ""

            data_output_pattern = f"Data/*-{div}*-{moon_str}*.simtel.zst"

            # substep 1 - verify the number of events in the simtel file
            mgv_step = self.setExecutable(
                "dirac_simtel_check",
                arguments=f"'{data_output_pattern}'",
                logFile=f"Verify_n_showers_{moon_str}_{div_str}Log.txt",
            )
            mgv_step["Value"]["name"] = f"Step{i_step}.{i_substep}_VerifyNShowers"
            mgv_step["Value"]["descr_short"] = "Verify number of showers"

            i_substep += 1

            # substep 2 - upload data file on SE and register in catalog
            dm_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
                % (
                    md_json,
                    file_md_json,
                    self.base_path,
                    data_output_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"DataManagement_{moon_str}_{div_str}Log.txt",
            )
            dm_step["Value"]["name"] = f"Step{i_step}.{i_substep}_DataManagement"
            dm_step["Value"][
                "descr_short"
            ] = "Save data files to SE and register them in DFC"
            i_step += 1

            # substep 3 - upload log and histo file on SE and register in catalog
            file_meta_data = {}
            file_md_json = json.dumps(file_meta_data)
            log_file_pattern = f"Data/*-{div}*-{moon_str}*.log_hist.tar"
            log_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' %s '%s' %s %s '%s' Log"
                % (
                    md_json,
                    file_md_json,
                    self.base_path,
                    log_file_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"LogManagement_{moon_str}_{div_str}Log.txt",
            )
            log_step["Value"]["name"] = f"Step{i_step}.{i_substep}_LogManagement"
            log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
            i_step += 1

        # Last step - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = f"Step{i_step}_LSHOME_End"
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv({"NSHOW": f"{self.n_shower}"})



#!/usr/bin/env python
"""
Launch a transformation from CWL workflow descriptions

Usage:
    cta-prod-submit-from-cwl <name of the Transformation> <type of the Transformation> <optional: dataset name> <optional: group_size>

Examples:
    cta-prod-submit-from-cwl Transformation_test MCSimulation
    cta-prod-submit-from-cwl Transformation_test Processing Prod5b_LaPalma_AdvancedBaseline_NSB1x_electron_North_20deg_R1 5
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Base.Script import Script
from CTADIRAC.Interfaces.API.MCPipeNSBJob import MCPipeNSBJob
from CTADIRAC.Interfaces.API.Prod5CtaPipeModelingJob import Prod5CtaPipeModelingJob
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ

import DIRAC
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from cwltool.load_tool import load_tool
from cwltool.context import LoadingContext, RuntimeContext
import cwltool.process
import cwltool.main
import schema_salad
import shellescape
import re
import copy
import os


class WorkflowStep:
    """Composite class for workflow step (production step + job)"""

    #############################################################################

    def __init__(self):
        """Constructor"""
        self.command_line = ""

    def get_command_line(self, input_cwl, input_yaml):
        """Get command line from a CommandLineTool description"""

        loader = schema_salad.ref_resolver.Loader(
            {"@base": input_yaml, "path": {"@type": "@id"}}
        )
        inputs, _ = loader.resolve_ref(input_yaml)

        for entry in inputs.values():
            if check_if_file_input(entry):
                add_fake_file(entry["location"])

        loading_context = LoadingContext({"strict": False, "debug": True})
        step = load_tool(input_cwl, loading_context)
        for inp in step.tool["inputs"]:
            if cwltool.process.shortname(inp["id"]) in inputs:
                pass
            elif (
                cwltool.process.shortname(inp["id"]) not in inputs and "default" in inp
            ):
                inputs[cwltool.process.shortname(inp["id"])] = copy.copy(inp["default"])
            elif (
                cwltool.process.shortname(inp["id"]) not in inputs
                and inp["type"][0] == "null"
            ):
                pass
            else:
                raise Exception(
                    f"Missing inputs `{cwltool.process.shortname(inp['id'])}`"
                )

        runtime_context = RuntimeContext()

        for job in step.job(inputs, None, runtime_context):
            for i in range(len(job.builder.bindings)):
                if isinstance(
                    job.builder.bindings[i]["datum"], dict
                ):  # replace file path by its location
                    if "location" in job.builder.bindings[i]["datum"]:
                        path = job.builder.bindings[i]["datum"]["path"]
                        location = job.builder.bindings[i]["datum"]["location"]
                        for k in range(len(job.command_line)):
                            if job.command_line[k] == path:
                                job.command_line[k] = location
            self.command_line = " ".join([maybe_quote(arg) for arg in job.command_line])

        for entry in inputs.values():
            if check_if_file_input(entry):
                remove_fake_file(entry["location"])
        return


def check_if_file_input(entry):
    if isinstance(entry, dict):
        if "class" in entry:
            if entry["class"] == "File":
                return True
    else:
        return False


def add_fake_file(location):
    dir = os.path.dirname(location)
    if dir != "":
        if not os.path.exists(dir):
            os.makedirs(dir)
    open(location, "a").close()
    return


def remove_fake_file(location):
    os.remove(location)
    return


def maybe_quote(arg):
    needs_shell_quoting = re.compile(r"""(^$|[\s|&;()<>\'"$@])""").search
    return shellescape.quote(arg) if needs_shell_quoting(arg) else arg


def submit_simulation_transformation(transfo_name):
    """Build MC Simulation Transformation"""

    # Build Transformation
    transfo = Transformation()
    transfo.Name = "MCSimulation"
    transfo.setTransformationName(transfo_name)  # this must be unique
    transfo.setType("MCSimulation")
    transfo.setDescription("Prod6 MC Pipe TS")
    transfo.setLongDescription("Prod6 simulation pipeline")  # mandatory

    # Get input files
    setup_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/setup-software.cwl"
    setup_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/setup-software.yml"

    run_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/dirac_prod_run.cwl"
    run_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/dirac_prod_run.yml"

    data_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/cta-user-managedata.cwl"
    data_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/cta-user-managedata.yml"

    # Build WorkflowSteps
    step1 = WorkflowStep()
    step1.get_command_line(setup_cwl, setup_yml)

    step2 = WorkflowStep()
    step2.get_command_line(run_cwl, run_yml)
    step2.command_line = (
        "./" + step2.command_line
    )  # add ./ since dirac_prod_run is not a command

    step3 = WorkflowStep()
    step3.get_command_line(data_cwl, data_yml)

    # Build Job
    MCJob = MCPipeNSBJob()
    MCJob.setType("MCSimulation")
    MCJob.setOutputSandbox(["*Log.txt"])

    # Replace static run number with dynamic run number to run with DIRAC
    step2.command_line = re.sub(
        "--run [0-9]+", f"--run {MCJob.run_number}", step2.command_line
    )

    # Run workflow
    i_step = 1
    sw_step = MCJob.setExecutable(
        step1.command_line,
        logFile="SetupSoftware_Log.txt",
    )
    sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
    sw_step["Value"]["descr_short"] = "Setup software"
    i_step += 1

    cs_step = MCJob.setExecutable(
        step2.command_line,
        logFile="CorsikaSimtel_Log.txt",
    )

    cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
    cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"

    i_step += 1
    dm_step = MCJob.setExecutable(
        step3.command_line,
        logFile="DataManagement_dark_Log.txt",
    )

    dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
    dm_step["Value"]["descr_short"] = "Save data files to SE and register them in DFC"

    MCJob.setExecutionEnv({"NSHOW": "10"})

    # Submit Transformation
    transfo.setBody(MCJob.workflow.toXML())
    result = transfo.addTransformation()  # transformation is created here
    if not result["OK"]:
        return result
    transfo.setStatus("Active")
    transfo.setAgentType("Automatic")
    trans_id = transfo.getTransformationID()
    return trans_id


def submit_processing_transformation(transfo_name, dataset, group_size):
    """Build Processing Transformation"""

    # Build Transformation
    transfo = Transformation()
    transfo.Name = "Prod5_ctapipe_processing"
    transfo.setTransformationName(transfo_name)  # this must be unique
    transfo.setType("DataReprocessing")
    transfo.setDescription("ctapipe Modeling TS")
    transfo.setLongDescription("ctapipe Modeling processing")  # mandatory

    # Get input files
    setup_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/setup-software.cwl"
    setup_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/setup-software-processing.yml"

    run_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/dirac_ctapipe-process_wrapper.cwl"
    run_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/dirac_ctapipe-process_wrapper.yml"

    data_cwl = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/cta-user-managedata.cwl"
    data_yml = "/home/afaure/Codes/CTADIRAC/src/CTADIRAC/ProductionSystem/CWL/cta-user-managedata-processing.yml"

    # Build WorkflowSteps
    step1 = WorkflowStep()
    step1.get_command_line(setup_cwl, setup_yml)

    step2 = WorkflowStep()
    step2.get_command_line(run_cwl, run_yml)
    step2.command_line = (
        "./" + step2.command_line
    )  # add ./ since dirac_ctapipe-process_wrapper is not a command

    step3 = WorkflowStep()
    step3.get_command_line(data_cwl, data_yml)

    input_meta_query = get_dataset_MQ(dataset)

    # Build Job
    ProcessingJob = Prod5CtaPipeModelingJob(cpuTime=259200.0)
    ProcessingJob.setOutputSandbox(["*Log.txt"])

    # Run workflow
    i_step = 1
    sw_step = ProcessingJob.setExecutable(
        step1.command_line,
        logFile="SetupSoftware_Log.txt",
    )
    sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
    sw_step["Value"]["descr_short"] = "Setup software"
    i_step += 1

    cs_step = ProcessingJob.setExecutable(
        step2.command_line,
        logFile="ctapipe_stage1_Log.txt",
    )

    cs_step["Value"]["name"] = "Step%i_ctapipe_stage1" % i_step
    cs_step["Value"]["descr_short"] = "Run ctapipe stage 1"

    i_step += 1
    dm_step = ProcessingJob.setExecutable(
        step3.command_line,
        logFile="DataManagement_Log.txt",
    )

    dm_step["Value"]["name"] = f"Step{i_step}_DataManagement"
    dm_step["Value"]["descr_short"] = "Save data files to SE and register them in DFC"

    # Submit Transformation
    transfo.setBody(ProcessingJob.workflow.toXML())
    transfo.setGroupSize(group_size)
    transfo.setInputMetaQuery(input_meta_query)
    result = transfo.addTransformation()  # transformation is created here
    if not result["OK"]:
        return result
    transfo.setStatus("Active")
    transfo.setAgentType("Automatic")
    trans_id = transfo.getTransformationID()
    return trans_id


@Script()
def main():
    Script.parseCommandLine()
    argss = Script.getPositionalArgs()
    if len(argss) not in [2, 4]:
        Script.showHelp()
    transfo_name = argss[0]
    type = argss[1]
    if len(argss) >= 3:
        dataset = argss[2]
        if len(argss) == 4:
            group_size = int(argss[3])
    if type.lower() == "mcsimulation":
        result = submit_simulation_transformation(transfo_name)
    elif type.lower() == "processing":
        result = submit_processing_transformation(transfo_name, dataset, group_size)
    try:
        if not result["OK"]:
            DIRAC.gLogger.error(result["Message"])
            DIRAC.exit(-1)
        else:
            DIRAC.gLogger.notice("Done")
    except Exception:
        DIRAC.gLogger.exception()
        DIRAC.exit(-1)


if __name__ == "__main__":
    main()



"""
   Wrapper around the job class to build a workflow element (production step + job)
"""

__RCSID__ = "$Id$"

# generic imports
from copy import deepcopy
import json

# DIRAC imports
from CTADIRAC.Interfaces.API.CtapipeMergeJob import CtapipeMergeJob
from CTADIRAC.ProductionSystem.Client.WorkflowElement import WorkflowElement


class MergingElement(WorkflowElement):
    """Composite class for workflow element (production step + job)"""

    #############################################################################

    def __init__(self, parent_prod_step):
        """Constructor"""
        WorkflowElement.__init__(self, parent_prod_step)
        self.job = CtapipeMergeJob(cpuTime=259200.0)
        self.job.setOutputSandbox(["*Log.txt"])
        self.job.input_limit = None
        self.job.merged = 0
        # self.job.output_extension = "merged.DL2.h5"
        self.prod_step.Type = "DataReprocessing"
        self.prod_step.Name = "Merging"
        self.mandatory_keys = {"MCCampaign", "configuration_id", "version"}
        self.constrained_job_keys = {"catalogs", "group_size", "moon"}
        self.constrained_input_keys = {
            "pointing_dir",
            "zenith_angle",
            "sct",
            "moon",
            "div_ang",
        }
        self.file_meta_fields = {"nsb", "div_ang", "split"}

    def set_constrained_job_attribute(self, key, value):
        """Set job attribute with constraints"""
        if key == "catalogs":
            # remove whitespaces between catalogs if there are some and separate between commas
            setattr(self.job, key, json.dumps(value.replace(", ", ",").split(sep=",")))
        elif key == "group_size":
            setattr(self.job, key, value)
            self.prod_step.GroupSize = self.job.group_size
        elif key == "moon":
            if value == "dark":
                self.job.output_file_metadata["nsb"] = 1
            elif value == "half":
                self.job.output_file_metadata["nsb"] = 5
            elif value == "full":
                self.job.output_file_metadata["nsb"] = 19

    def set_constrained_input_query(self, key, value):
        """Set input meta query with constraints"""
        if key == "pointing_dir":
            if value == "North":
                self.prod_step.Inputquery["phiP"] = 180
            elif value == "South":
                self.prod_step.Inputquery["phiP"] = 0
        elif key == "zenith_angle":
            self.prod_step.Inputquery["thetaP"] = float(value)
        elif key == "sct":
            self.prod_step.Inputquery["sct"] = str(value)
        elif key == "moon":
            if value == "dark":
                self.prod_step.Inputquery["nsb"] = 1
            elif value == "half":
                self.prod_step.Inputquery["nsb"] = 5
            elif value == "full":
                self.prod_step.Inputquery["nsb"] = 19
        elif key == "div_ang":
            self.prod_step.Inputquery["div_ang"] = str(value)

    def build_job_output_data(self, workflow_step):
        """Build job output meta data"""
        metadata = deepcopy(self.prod_step.Inputquery)
        for key, value in workflow_step["job_config"].items():
            metadata[key] = value
        merged = self.get_merging_level()
        metadata["merged"] = merged
        self.job.set_output_metadata(metadata)

    def get_merging_level(self):
        """Get the merging level from parent step or from the user query"""
        if self.prod_step.ParentStep:
            merged = self.prod_step.ParentStep.Outputquery["merged"]
        else:
            merged = self.job.merged
        return merged

    def build_element_config(self):
        """Set job and production step attributes specific to the configuration"""
        self.prod_step.GroupSize = self.job.group_size
        self.job.set_executable_sequence(debug=False)
        self.prod_step.Body = self.job.workflow.toXML()

    #############################################################################



"""
  New job class to run Prod6 single-telescope muon simulations
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
import numpy as np

# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class Prod6MCPipeMuonJob(Job):
    """Job extension class for Prod5 MC NSB simulations,
    takes care of running corsika piped into simtel
    2 output files are created for Dark and Moon NSB
    """

    def __init__(self, cpu_time=259200):
        """Constructor takes almosst everything from base class

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpu_time)
        self.setName("Prod6MC_Generation")
        self.setType("MCSimulation")
        self.package = "corsika_simtelarray"
        self.version = "2022-08-03-sc"
        self.compiler = "gcc83_matchcpu"
        self.program_category = "tel_sim"
        self.prog_name = "sim_telarray"
        self.configuration_id = 15
        self.output_data_level = DATA_LEVEL_METADATA_ID["R1"]
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD6/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.metadata = collections.OrderedDict()

    def set_site(self, site):
        """Set the site to simulate

        Parameters:
        site -- a string for the site name (LaPalma)
        """
        if site in ["Paranal", "LaPalma"]:
            DIRAC.gLogger.info(f"Set Corsika site to: {site}")
            self.cta_site = site
        else:
            DIRAC.gLogger.error(f"Site is unknown: {site}")
            DIRAC.exit(-1)

    def set_telescope(self, telescope):
        """Set the telescope to simulate

        Parameters:
        telescope -- a string for the telescope name
        """
        if telescope in ["LST", "MST-FlashCam", "MST-NectarCam", "SST", "SCT"]:
            DIRAC.gLogger.info(f"Set telescope to: {telescope}")
            self.telescope = telescope
        else:
            DIRAC.gLogger.error(f"Telescope {telescope} is unknown, aborting")
            DIRAC.exit(-1)

    def set_particle(self, particle):
        """Set the corsika primary particle

        Parameters:
        particle -- a string for the particle type/name
        """
        if particle in ["muon"]:
            DIRAC.gLogger.info(f"Set Corsika particle to: {particle}")
            self.particle = particle
        else:
            DIRAC.gLogger.error(
                "This script can only perform muon simulations, not %s simulations"
                % particle
            )
            DIRAC.exit(-1)

    def set_pointing_dir(self, pointing):
        """Set the pointing direction, North or South

        Parameters:
        pointing -- a string for the pointing direction
        """
        if pointing in ["North", "South", "East", "West"]:
            DIRAC.gLogger.info(f"Set Pointing dir to: {pointing}")
            self.pointing_dir = pointing
        else:
            DIRAC.gLogger.error(f"Unknown pointing direction: {pointing}")
            DIRAC.exit(-1)

    def set_half_moon(self, half_moon=False):
        """Set if to simulate with full-moon conditions

        Parameters:
        half_moon -- a boolean for simulating with half-moon conditions
        """
        if half_moon is True:
            DIRAC.gLogger.info("Set simulations with half-moon conditions")
            self.half_moon = "--with-half-moon"
        else:
            self.half_moon = ""

    def set_degraded_mirror_reflectivity(self, degraded_mirror_reflectivity=False):
        """Set the values of degraded mirror reflectivity to simulate

        Parameters:
        degraded_mirror_reflectivity -- a boolean to set if to simulate a degraded mirror reflectivity (0.3 to 1.0 in steps of 0.05)
        """
        if degraded_mirror_reflectivity:
            DIRAC.gLogger.info("Set simulations with degraded mirror reflectivities")
            self.degraded_values = np.arange(0.3, 1.05, 0.05)
            self.degraded_mirror = "--degraded-mirror-ref"
        else:
            DIRAC.gLogger.info("Set simulations without degraded mirror reflectivities")
            self.degraded_values = [1]  # No degraded mirror reflectivity
            self.degraded_mirror = ""

    def set_meta_data(self):
        """define the common meta data of the application"""
        # The order of the metadata dictionary is important,
        # since it's used to build the directory structure
        self.metadata["array_layout"] = "Single-telescope"
        self.metadata["site"] = self.cta_site
        self.metadata["particle"] = self.particle
        # for air shower simulation means North=0 and South=180
        # but here piped into tel_sim so North=180 and South=0
        if self.pointing_dir == "North":
            self.metadata["phiP"] = 180
        if self.pointing_dir == "South":
            self.metadata["phiP"] = 0
        self.metadata["thetaP"] = float(self.zenith_angle)
        self.metadata[self.program_category + "_prog"] = self.prog_name
        self.metadata[self.program_category + "_prog_version"] = "".join(
            self.version.rsplit("-sc", 1)
        )
        self.metadata["data_level"] = self.output_data_level
        self.metadata["configuration_id"] = self.configuration_id
        if self.telescope == "SCT":
            self.metadata["sct"] = "True"
        else:
            self.metadata["sct"] = "False"

    def setupWorkflow(self, debug=False):
        """Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
        """
        # step 1 - debug only
        i_step = 1
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2 : use new CVMFS repo
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s -r /cvmfs/sw.cta-observatory.org/software"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run corsika+sim_telarray
        prod_exe = "./dirac_prod6_muon_run"
        prod_args = "--start_run {} --run {} {} {} {} {} {} {} {}".format(
            self.start_run_number,
            self.run_number,
            self.half_moon,
            self.degraded_mirror,
            self.telescope,
            self.cta_site,
            self.particle,
            self.pointing_dir,
            self.zenith_angle,
        )

        cs_step = self.setExecutable(
            prod_exe, arguments=prod_args, logFile="CorsikaSimtel_Log.txt"
        )
        cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
        cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"
        i_step += 1

        # step 4 - define meta data, upload file on SE and register in catalogs
        self.set_meta_data()
        md_json = json.dumps(self.metadata)

        i_step += 1
        i_substep = 0
        for degraded in self.degraded_values:
            data_output_pattern = f"Data/*dark-ref-degraded-{degraded:.2f}.simtel.zst"

            # step 5 verify the number of events in the simtel file
            mgv_step = self.setExecutable(
                "dirac_simtel_check",
                arguments=f"'{data_output_pattern}'",
                logFile="Verify_n_showers_Log.txt",
            )
            mgv_step["Value"]["name"] = f"Step{i_step}.{i_substep}_VerifyNShowers"
            mgv_step["Value"]["descr_short"] = "Verify number of showers"

            # Upload and register data - NSB=1 dark
            file_meta_data = {"runNumber": self.run_number, "nsb": 1}
            file_md_json = json.dumps(file_meta_data)

            dm_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
                % (
                    md_json,
                    file_md_json,
                    self.base_path,
                    data_output_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"DataManagement_dark_ref_degraded_{degraded:.2f}Log.txt",
            )
            dm_step["Value"]["name"] = f"Step{i_step}.{i_substep}_DataManagement"
            dm_step["Value"][
                "descr_short"
            ] = "Save data files to SE and register them in DFC"
            i_substep += 1

            # Upload and register log and histo file - NSB=1
            file_meta_data = {}
            file_md_json = json.dumps(file_meta_data)
            log_file_pattern = f"Data/*dark-ref-degraded-{degraded:.2f}.log_hist.tar"
            log_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
                % (
                    md_json,
                    md_field_json,
                    file_md_json,
                    self.base_path,
                    log_file_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile=f"LogManagement_dark_ref_degraded_{degraded:.2f}Log.txt",
            )
            log_step["Value"]["name"] = f"Step{i_step}.{i_substep}_LogManagement"
            log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
            i_substep += 1

            if self.half_moon == "--with-half-moon":
                # Now switching to half moon NSB
                # Upload and register data - NSB=5 half moon
                file_meta_data = {"runNumber": self.run_number, "nsb": 5}
                file_md_json = json.dumps(file_meta_data)
                data_output_pattern = "Data/*moon-ref-degraded-%.2f.simtel.zst" % (
                    degraded
                )

                dm_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' %s '%s' %s %s '%s' Data"
                    % (
                        md_json,
                        file_md_json,
                        self.base_path,
                        data_output_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile=f"DataManagement_moon_ref_degraded_{degraded:.2f}Log.txt",
                )
                dm_step["Value"]["name"] = f"Step{i_step}.{i_substep}_DataManagement"
                dm_step["Value"][
                    "descr_short"
                ] = "Save data files to SE and register them in DFC"
                i_substep += 1

                # Upload and register log file - NSB=5
                file_meta_data = {}
                file_md_json = json.dumps(file_meta_data)
                log_file_pattern = "Data/*moon-ref-degraded-%.2f.log_hist.tar" % (
                    degraded
                )
                log_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' %s '%s' %s %s '%s' Log"
                    % (
                        md_json,
                        file_md_json,
                        self.base_path,
                        log_file_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile=f"LogManagement_moon_ref_degraded_{degraded:.2f}Log.txt",
                )
                log_step["Value"]["name"] = f"Step{i_step}.{i_substep}_LogManagement"
                log_step["Value"][
                    "descr_short"
                ] = "Save log to SE and register them in DFC"
                i_substep += 1

            i_step += 1

        # Step 6 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = f"Step{i_step}_LSHOME_End"
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv(
            {
                "NSHOW": f"{self.n_shower}",
                "CORSIKA_MULTIPLEX_SEQUENTIAL": "1",  # Set CORSIKA multipipe to run in sequential mode (better for single-slot/cpu grid jobs)
            }
        )



"""
  Simple Wrapper on the Job class to handle ctapipe processing applications
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
from DIRAC.Interfaces.API.Job import Job


class ProcessingJob(Job):
    """Job extension class for ctapipe processing"""

    def __init__(self, cpuTime=432000):
        """Constructor

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        super().__init__()
        self.setCPUTime(cpuTime)
        # defaults
        self.setName("ctapipe")
        self.package = "ctapipe"
        self.version = "v0.10.0"
        self.compiler = "gcc48_default"
        self.program_category = "analysis"
        self.output_metadata = collections.OrderedDict()
        self.output_file_metadata = collections.OrderedDict()
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        # to be redefined in the derived classes
        self.prog_name = "ctapipe-process"
        self.configuration_id = 1
        self.output_data_level = 1
        self.MCCampaign = "ProdTest"

    def set_output_metadata(self, metadata):
        """Set output metadata

        Parameters:
        metadata -- metadata dictionary from telescope simulation
        """
        # # Set directory meta data
        self.output_metadata = collections.OrderedDict()
        self.output_metadata["array_layout"] = metadata["array_layout"]
        self.output_metadata["site"] = metadata["site"]
        self.output_metadata["particle"] = metadata["particle"]
        try:
            phiP = metadata["phiP"]["="]
        except BaseException:
            phiP = metadata["phiP"]
        self.output_metadata["phiP"] = phiP
        try:
            thetaP = metadata["thetaP"]["="]
        except BaseException:
            thetaP = metadata["thetaP"]
        self.output_metadata["thetaP"] = thetaP
        if metadata.get("sct"):
            self.output_metadata["sct"] = metadata["sct"]
        else:
            self.output_metadata["sct"] = "False"
        self.output_metadata[self.program_category + "_prog"] = self.prog_name
        self.output_metadata[self.program_category + "_prog_version"] = self.version
        self.output_metadata["data_level"] = self.output_data_level
        self.output_metadata["configuration_id"] = self.configuration_id
        self.output_metadata["merged"] = 0
        self.output_metadata["MCCampaign"] = self.MCCampaign
